 /**    
  *******************************************************************************************************   
  * @attention    
  * Copyright (c) 2023, SHANGHAI FUDAN MICROELECTRONICS GROUP CO., LTD.(FUDAN MICROELECTRONICS./ FUDAN MICRO.)    
  * All rights reserved.    
  *    
  * Processor:                   FM33LF0xxA    
  * http:                        http://www.fmdevelopers.com.cn/    
  *    
  * Redistribution and use in source and binary forms, with or without    
  * modification, are permitted provided that the following conditions are met    
  *    
  * 1. Redistributions of source code must retain the above copyright notice,    
  *    this list of conditions and the following disclaimer.    
  *    
  * 2. Redistributions in binary form must reproduce the above copyright notice,    
  *    this list of conditions and the following disclaimer in the documentation    
  *    and/or other materials provided with the distribution.    
  *    
  * 3. Neither the name of the copyright holder nor the names of its contributors    
  *    may be used to endorse or promote products derived from this software    
  *    without specific prior written permission.    
  *    
  * 4. To provide the most up-to-date information, the revision of our documents     
  *    on the World Wide Web will be the most Current. Your printed copy may be      
  *    an earlier revision. To verify you have the latest information avaliable,    
  *    refer to: http://www.fmdevelopers.com.cn/.    
  *    
  * THIS SOFTWARE IS PROVIDED BY FUDAN MICRO "AS IS" AND ANY EXPRESSED:     
  * ORIMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES     
  * OF MERCHANTABILITY NON-INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE    
  * ARE DISCLAIMED.IN NO EVENT SHALL FUDAN MICRO OR ITS CONTRIBUTORS BE LIABLE     
  * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL     
  * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS     
  * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER    
  * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,     
  * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISINGIN ANY WAY OUT OF THE     
  * USE OF THIS SOFTWARE, EVEN IF ADVISED OFTHE POSSIBILITY OF SUCH DAMAGE.  
  *
  ******************************************************************************************************* 
  */ 

#include "main.h"
#include "lin.h"
#include "uart.h"

#define FRAME_DATA_MAX_LEN          11  /* 帧最大数据长度 */
#define SCHEDULE_FRAME_MAX_NUM      255 /* 进度表最大帧数量 */

#define FRAME_TYPE_UNCONDITIONAL    0   /* 无条件帧 */
#define FRAME_TYPE_EVENT_TRIGGER    1   /* 事件触发帧 */
#define FRAME_TYPE_SPORADIC         2   /* 偶发帧 */
#define FRAME_TYPE_DIAGNOSTIC       3   /* 诊断帧 */
#define FRAME_TYPE_RESERVED         4   /* 保留帧 */
#define FRAME_TYPE_NO_RESPONSE      5   /* 自定义，主机发送无需回应帧 */

/* 帧处理状态机 */
#define FRAME_STATE_IDLE            0   /* 空闲状态 主：等待任务开始 从：固定波特率：等待间隔段 自动波特率：等待间隔段+同步段 */
#define FRAME_STATE_SEND_SYN        1   /* 主机发送SYN(包括间隔段和同步段) */
#define FRAME_STATE_RECV_SYN        2   /* 固定波特率：接收同步段0x55 */
#define FRAME_STATE_RECV_PID        3   /* 接收PID */
#define FRAME_STATE_RECV_DATA       4   /* 接收数据 */
#define FRAME_STATE_SEND            5   /* 串口发送 */
#define FRAME_STATE_SLEEP           6   /* 休眠状态 */

uint32_t Enter_Critical( void )    
{
    uint32_t reg;
    reg=__get_PRIMASK();
    __set_PRIMASK(1);
    return reg;
} 
void Exit_Critical( uint32_t reg )    
{
    __set_PRIMASK(reg);
     return ;
}

#define START_CRITICAL()    unsigned int reg;
#define ENTER_CRITICAL()    reg = Enter_Critical()
#define EXIT_CRITICAL()     Exit_Critical(reg)

#if (SLEEP_SUPPORT == 1)
extern uint8_t  g_awake_init;

void lin_sleep_init(void);
#endif

/**
  * @brief  计算PID
  * @param  id: 输入id
  * @retval 计算后的PID 
  */
uint8_t calculatePID(uint8_t id)
{
    uint8_t P0 = 0, P1 = 0;

    P0 = ((id) ^ (id >> 1) ^ (id >> 2) ^ (id >> 4)) & 0x01;
    P1 = (~((id >> 1) ^ (id >> 3) ^ (id >> 4) ^ (id >> 5))) & 0x01;

    return ((P1 << 7) | (P0 << 6) | id);
}

/**
  * @brief  计算校验和
  * @param  data: 数据指针
  *         len:  数据长度
  * @retval checkSum: 校验和 
  */
uint8_t calculateChecksum(uint8_t *data, uint8_t len)
{
    uint8_t i;
    uint16_t checkSum = 0;

    for (i = 0; i < len; i++)
    {
        checkSum += data[i];
        if (checkSum >= 0x100)
        {
            checkSum -= 0xFF;
        }
    }
    checkSum = (~checkSum) & 0xFF;

    return checkSum;
}

/**
  * @brief  LIN slp初始化
  * @param  void
  * @retval void
  */
void lin_slp_pin_init(void)
{
    FL_GPIO_InitTypeDef GPIO_InitStruct = {0};

    LIN_SLP_DISABLE();

    GPIO_InitStruct.pin           = LIN_SLP_PIN;
    GPIO_InitStruct.mode          = FL_GPIO_MODE_OUTPUT;
    GPIO_InitStruct.outputType    = FL_GPIO_OUTPUT_PUSHPULL;
    GPIO_InitStruct.driveStrength = FL_GPIO_DRIVESTRENGTH_X3;
    GPIO_InitStruct.pull          = FL_GPIO_BOTH_DISABLE;
    GPIO_InitStruct.remapPin      = FL_GPIO_PINREMAP_FUNCTON1;
    FL_GPIO_Init(LIN_SLP_GPIO, &GPIO_InitStruct);
}


#define Min(a, b) (((a)<(b))?(a):(b))
#define lin_calc_max_res_timeout_cnt(bitLength, size) (uint16_t)((14*(1+(size))*((bitLength)/TIMER_1US_LOAD))/TIME_BASE_PERIOD + 3 )

#if (AUTOBAUD == 1)

/* lin波特率和字节间间隔 */
#define LIN_BAUDRATE_MAX                20000  /* Lin支持的最大波特率 */
#if (MAIN_CLOCK == 16000000)                   /* 16MHz下对应BGR */
#define LIN_BAUDRATE_9600_BGR           1666                            
#define LIN_BAUDRATE_19200_BGR          832                             
#define LIN_SYN_BAUDRATE_MAX_BGR        39     /* 400K  */
#elif (MAIN_CLOCK == 24000000)                 /* 24MHz下对应BGR */
#define LIN_BAUDRATE_9600_BGR           2499 
#define LIN_BAUDRATE_19200_BGR          1249                            
#define LIN_SYN_BAUDRATE_MAX_BGR        59     /* 400K  */
#elif (MAIN_CLOCK == 32000000)                 /* 32MHz下对应BGR */
#define LIN_BAUDRATE_9600_BGR           3332 
#define LIN_BAUDRATE_19200_BGR          1666                            
#define LIN_SYN_BAUDRATE_MAX_BGR        79     /* 400K  */
#else                                          /* 8MHz下对应BGR  */
#define LIN_BAUDRATE_9600_BGR           832 
#define LIN_BAUDRATE_19200_BGR          416                             
#define LIN_SYN_BAUDRATE_MAX_BGR        19     /* 400K */
#endif /* End (MCU_BUS_FREQ == 16000000) */

uint32_t    BitLength = LIN_BAUDRATE_19200_BGR; /* 位速率对应的BGR */

#endif

/* 帧格式 [最大11字节] */
/* data区内容：受保护ID-1字节; 数据段-0-8字节；校验和-1字节 */
typedef struct s_lin_frame_type
{
    uint8_t tx_len;/* 发送长度 */
    uint8_t rx_len;/* 接收长度 */
    uint8_t data[FRAME_DATA_MAX_LEN];
} lin_frame_data_type;

/* 任务处理：等待接收并处理 */
typedef struct s_lin_task_type
{
    uint8_t frame_state;  /* 帧处理状态机，参照 FRAME_STATE_IDLE 定义 */

    /* 帧超时计数，接收到同步段后置起，按照8字节数据计算超时 */
    /* 公式：round((1.4x(10+8x10)xTbit)/Tbase_period) + 3，8为接收字节数，Tbit为单个bit用时，Tbase_period为定时器周期，单位都是us */
    uint16_t frame_timeout_cnt;
    /* 响应超时计数，接收到需要响应的ID后置起，按照实际接收长度计算超时 */
    /* 公式：round((1.4x(10+Nx10)xTbit)/Tbase_period) + 3，N为接收字节数，Tbit为单个bit用时，Tbase_period为定时器周期，单位都是us */
    uint16_t res_frame_timeout_cnt;
    /* Lin总线IDLE超时计数，减为0进入SLEEP状态 */
    uint16_t idle_timeout_cnt;
    lin_frame_data_type lin_frame;/* 帧内容 */
} lin_task_type;

lin_task_type g_lin_task;

uint8_t LIN_CMD0_Data[8];
uint8_t LIN_CMD1_Data[8] = {0x12, 0x34, 0x56, 0x78};
uint8_t LIN_CMD2_Data[8] = {0x11, 0x22, 0x33, 0x44, 0x55, 0x66, 0x77, 0x88};

lin_cmd_packet_t scheduleTable[] =
{
    /* 用户命令码 */
    {LIN_CMD0, RECEIVE,  LIN_CMD0_LENGTH, LIN_CMD0_Data},
    {LIN_CMD1, TRANSMIT, LIN_CMD1_LENGTH, LIN_CMD1_Data},
    {LIN_CMD2, TRANSMIT, LIN_CMD2_LENGTH, LIN_CMD2_Data},
};

/**
  * @brief  接收超时定时器初始化
  * @param  void
  * @retval void
  */
void lin_timeout_init(void)
{
    /* 使能TAU时钟 */
    FL_CMU_EnableGroup4BusClock(FL_CMU_GROUP4_BUSCLK_TAU0);

    /* 配置TAU00普通计数模式 */
    FL_TAU_SetOperationMode(TAU00, FL_TAU_OPERATION_MODE_NORMAL);
    FL_TAU_WritePrescaler(TAU00, 0); 
    FL_TAU_WriteAutoReload(TAU00, MAIN_CLOCK/1000000 * TIME_BASE_PERIOD - 1);    
    
    /* NVIC中断优先级 */
    NVIC_DisableIRQ(TAU0_IRQn);
    NVIC_SetPriority(TAU0_IRQn, LIN_TIMER_IRQ_PRI);
    NVIC_EnableIRQ(TAU0_IRQn);
    
    FL_TAU_EnableIT_Update(TAU00);
    FL_TAU_Enable(TAU00);  /* 使能定时器 */
}


/**
  * @brief  lin idle
  * @param  void
  * @retval void
  */
void lin_goto_idle(void)
{
    START_CRITICAL();

    ENTER_CRITICAL();

    g_lin_task.lin_frame.tx_len = 0;
    g_lin_task.lin_frame.rx_len = 0;
    g_lin_task.frame_state = FRAME_STATE_IDLE;
    g_lin_task.idle_timeout_cnt = LIN_IDLE_TIMEOUT;
    
    UARTxOp.RxLen = 0;
    UARTxOp.TxLen = 0;
    
    #if (AUTOBAUD == 1)
    /* 关闭同步段中断 */
    LIN_UART->ISR = (UART_ISR_SER_Msk | UART_ISR_LINSYNCF_Msk);     /* 清同步段标志 */
    LIN_UART->IER &= ~UART_IER_LINSYNCIE_Msk;  /* 关同步段中断 */
    
    /* 切换波特率为初始最高波特率 */
//    LIN_UART->CSR &= ~UART_CSR_RXEN_Msk;
    LIN_UART->BGR = BitLength - 1;
//    LIN_UART->CSR |= UART_CSR_RXEN_Msk;
    #else
    FL_UART_EnableRX(LIN_UART);                /* 非自动波特率打开接收 */
    #endif

    EXIT_CRITICAL();
}


/**
  * @brief  lin idle state初始化
  * @param  void
  * @retval void
  */
__STATIC_INLINE void lin_init_idle_state(void)
{        
    g_lin_task.lin_frame.tx_len = 0;
    g_lin_task.lin_frame.rx_len = 0;
    g_lin_task.frame_state = FRAME_STATE_IDLE;
    g_lin_task.idle_timeout_cnt = LIN_IDLE_TIMEOUT;
    
    UARTxOp.RxLen = 0;
    UARTxOp.TxLen = 0;
}


/**
  * @brief  定时，用于判断字节间超时
  * @note   12位波特率，如1200-10ms 19.2KHz-625us
  * @param  void
  * @retval void
  */
void TAU0_IRQHandler(void)
{
    if (FL_TAU_IsActiveFlag_Update(TAU00))
    {
        FL_TAU_ClearFlag_Update(TAU00);

        switch (g_lin_task.frame_state)
        {
            case FRAME_STATE_IDLE:
                if (0 == g_lin_task.idle_timeout_cnt)
                {
                    g_lin_task.frame_state = FRAME_STATE_SLEEP;
                    g_lin_task.idle_timeout_cnt = LIN_IDLE_TIMEOUT;
                }
                else
                {
                    g_lin_task.idle_timeout_cnt--;
                }
                break;
            case FRAME_STATE_RECV_SYN:
            case FRAME_STATE_RECV_PID:
            case FRAME_STATE_SEND:
                if (0 == g_lin_task.frame_timeout_cnt)
                {
                    lin_goto_idle();
                }
                else
                {
                    g_lin_task.frame_timeout_cnt--;
                }
                break;
            case FRAME_STATE_RECV_DATA:
                if (0 == g_lin_task.res_frame_timeout_cnt)
                {
                    
                    if (UARTxOp.RxLen > 1)
                    {
                        UARTxOp.RxLen = 0;
                    }
                    lin_goto_idle();
                }
                else
                {
                    g_lin_task.res_frame_timeout_cnt--;
                }
                break;
        }
        
    }
}
/**
  * @brief  UART-LIN中断
  * @param  void
  * @retval void
  */
void LIN_UART_IRQHandler(void)
{
    uint8_t tmp08;
    static uint8_t i;
    
    #if (AUTOBAUD == 1)
    uint32_t isr_flag = 0;
    uint32_t tmp_32;
    static uint8_t init_flag = 0;    
    
    isr_flag = LIN_UART->ISR;
    
    /******************************
    *** 0. WAKEUP DETECTED
    *******************************/ 
    if ((isr_flag & UART_ISR_LINWKDF_Msk) &&
        (FL_SET == FL_UART_IsEnabledIT_LINWK(LIN_UART)))/* 收到唤醒信号 */
    {
        #if (SLEEP_SUPPORT == 1)
        if (g_awake_init == 1)
        {
            g_awake_init = 0;
            /* lin唤醒后配置 */
            lin_awake_init();
        }
        #endif

        FL_UART_ClearFlag_WakeupDetect(LIN_UART);  /* 清除唤醒信号检测标志 */
        FL_UART_DisableIT_LINWK(LIN_UART);         /* 关闭唤醒中断 */
        FL_UART_DisableLINWkupDetect(LIN_UART);    /* 关闭唤醒检测 */
        
        if (init_flag == 0)
        {
            lin_goto_idle();
            init_flag = 1;
        }

        LIN_UART->ISR = LIN_UART->ISR;
        return;
    }

    if ((isr_flag & UART_ISR_LINBF_Msk) &&
        (LIN_UART->IER & UART_IER_LINBIE_Msk)) /* 收到break段 */
    {
        /* Clear flag */
        LIN_UART->ISR = UART_ISR_LINBF_Msk;    /* 清break段接收完成标志 */
       
        g_lin_task.frame_timeout_cnt = lin_calc_max_res_timeout_cnt(BitLength, 8);/* 收到break后超时切换为较大值 */
        
        LIN_UART->ISR = (UART_ISR_SER_Msk | UART_ISR_LINSYNCF_Msk);/* 清同步段标志和同步段错误标志 */
        LIN_UART->IER |= UART_IER_LINSYNCIE_Msk;/* 开同步段中断，等待接收同步段0x55 */
        
        g_lin_task.frame_state = FRAME_STATE_RECV_SYN;  
        
        LIN_UART->ISR = LIN_UART->ISR;
        return;
    }
    
    /******************************
    *** 1.1 SYN DETECTED
    *******************************/
    if ((isr_flag & UART_ISR_LINSYNCF_Msk) &&
        (LIN_UART->IER & UART_IER_LINSYNCIE_Msk))  /* 收到同步段 */
    { 
        if (LIN_UART->ISR & UART_ISR_SER_Msk)
        {
            LIN_UART->ISR = (UART_ISR_SER_Msk | UART_ISR_LINSYNCF_Msk);
            
            lin_goto_idle();
            
            LIN_UART->ISR = LIN_UART->ISR;
        }
        else
        {
            LIN_UART->ISR = UART_ISR_LINSYNCF_Msk;     /* 清同步段标志 */
            LIN_UART->IER &= ~UART_IER_LINSYNCIE_Msk;  /* 关同步段中断 */
            
            /* 计算分频系数 */
            tmp_32 = LIN_UART->LINBSR;  /* 获取同步段计数时钟个数 */
            tmp_32 = (tmp_32 >> 3);

            BitLength = tmp_32;
            g_lin_task.lin_frame.rx_len = 0;
            g_lin_task.frame_state = FRAME_STATE_RECV_PID;
            UARTxOp.RxLen = 0;

        }
        LIN_UART->ISR = LIN_UART->ISR;
        return;
    }
    #else
    #if (SLEEP_SUPPORT == 1)
     if ((LIN_UART->ISR & UART_ISR_LINWKDF_Msk) &&
        (FL_SET == FL_UART_IsEnabledIT_LINWakeUp(LIN_UART))) /* 收到唤醒信号 */
    {   
        if (g_awake_init == 1)
        {
            g_awake_init = 2;

            LIN_UART->MCR = 0;         /* 配置为普通UART */

            /* lin唤醒后配置 */
            lin_awake_init();
        }
        
        FL_UART_ClearFlag_LINDetectWakeUpSignal(LIN_UART);    /* 清除唤醒信号检测标志 */
        FL_UART_DisableIT_LINWakeUp(LIN_UART);                /* 关闭唤醒中断 */
        FL_UART_DisableLINWkupDetect(LIN_UART);               /* 关闭唤醒检测 */
       
        LIN_UART->ISR = LIN_UART->ISR;
        return;
    }
    #endif
    #endif
    
    /* 接收错误中断处理 需要注意帧类型错误会同时有RXBF标志，用于判定同步段 */
    if (LIN_UART->ISR & UART_ISR_FERR_Msk)
    {
        tmp08 = LIN_UART->RXBUF;
        LIN_UART->ISR = UART_ISR_FERR_Msk;
        
        #if (AUTOBAUD == 1)
        lin_goto_idle();
        #else
        if (tmp08 == LIN_BREAK_BYTE)
        {
            /* 检测到间隔段 */
            g_lin_task.frame_state = FRAME_STATE_RECV_SYN;  
            
            /* 收到间隔段之后置起超时，等待在超时内收到同步字节0x55 */
            g_lin_task.frame_timeout_cnt = lin_calc_max_res_timeout_cnt(LIN_BIT_TIMER_LOAD, 8);
        }
        else
        {
            /* 帧错误 */
            
            lin_goto_idle();
        }  
        #endif
    }
    /* 接收中断处理 */
    else if (LIN_UART->ISR & UART_ISR_RXBF_Msk)
    {
        switch (g_lin_task.frame_state)
        {
            /* 固定波特率接收同步阶段 */
            case FRAME_STATE_RECV_SYN:
                /* 接收数据 */
                /* 接收中断标志可通过读取rxreg寄存器清除 */
                tmp08 = LIN_UART->RXBUF;
                UARTxOp.RxLen = 0;
            
                #if (SLEEP_SUPPORT == 1)
                if (g_awake_init == 2)
                {
                    /* 重置超时部分 */
                    g_awake_init = 0;
                    g_lin_task.frame_timeout_cnt = lin_calc_max_res_timeout_cnt(LIN_BIT_TIMER_LOAD, 8);
                }
                #endif           
            
                if (tmp08 == LIN_SYN_BYTE)
                {
                    /* 间隔段之后的第一个字节是0x55 */
                    g_lin_task.frame_state = FRAME_STATE_RECV_PID;
                }
                else
                {
                    lin_goto_idle();
                }
                break;
            /* 接收PID阶段，判断PID是否有效如有效判定后续行为是发送还是接收 */
            case FRAME_STATE_RECV_PID:
                /* 接收数据 */
                /* 接收中断标志可通过读取rxreg寄存器清除 */
                tmp08 = LIN_UART->RXBUF;
            
                UARTxOp.RxBuf[UARTxOp.RxLen] = tmp08;
                UARTxOp.RxLen++;
            
            
                for (i = 0; i < TABLE_SIZE; i++)
                {
                    if (scheduleTable[i].pid == tmp08)
                    {
                        break;
                    }
                }
                
                
                if (i == TABLE_SIZE)
                {
                    /* 无对应命令，不做处理 */
                    lin_goto_idle();

                    return;
                }
                else
                {
                    if (scheduleTable[i].type == RECEIVE)
                    {
                        /* 主机发送命令，更新待接收长度，转接收数据阶段 */
                        g_lin_task.lin_frame.rx_len = scheduleTable[i].length + 1;
                        g_lin_task.frame_state = FRAME_STATE_RECV_DATA;
                        
                        /* 更新响应超时 */
                        #if (AUTOBAUD == 1)
                        g_lin_task.res_frame_timeout_cnt = lin_calc_max_res_timeout_cnt(BitLength, scheduleTable[i].length);
                        #else
                        g_lin_task.res_frame_timeout_cnt = lin_calc_max_res_timeout_cnt(LIN_BIT_TIMER_LOAD, scheduleTable[i].length);
                        #endif
                    }
                    else
                    {
                        if (0x0 == scheduleTable[i].FullFlg)
                        {
                            /* 无数据发送 */
                            lin_goto_idle();
                            return;
                        }
                        
                        /* 主机接收命令，更新待发送长度，转发送数据阶段 */
                        /* 第一时间发送数据，保证负载率 */
                        LIN_UART->TXBUF = scheduleTable[i].data[0];
                        
                        #if (CHECKSUM_TYPE == CHECKSUM_TYPE_ENHANCED)
                            g_lin_task.lin_frame.data[0] = PID;
                            memcpy(&g_lin_task.lin_frame.data[1], scheduleTable[i].data, scheduleTable[i].length);
                            g_lin_task.lin_frame.data[scheduleTable[i].length + 1] = calculateChecksum(g_lin_task.lin_frame.data, scheduleTable[i].length + 1);

                            g_lin_task.lin_frame.tx_len = scheduleTable[i].length + 1;
                            UARTxOp.TxLen = g_lin_task.lin_frame.tx_len;
                            UARTxOp.TxOpc = 1;
                            UARTxOp.TxBuf = &g_lin_task.lin_frame.data[1];
                        #else
                            memcpy(g_lin_task.lin_frame.data, scheduleTable[i].data, scheduleTable[i].length);
                            g_lin_task.lin_frame.data[scheduleTable[i].length] = calculateChecksum(g_lin_task.lin_frame.data, scheduleTable[i].length);

                            g_lin_task.lin_frame.tx_len = scheduleTable[i].length + 1;
                            UARTxOp.TxLen = g_lin_task.lin_frame.tx_len;
                            UARTxOp.TxOpc = 1;
                            UARTxOp.TxBuf = g_lin_task.lin_frame.data;
                        #endif
                        g_lin_task.lin_frame.rx_len = 0;
                        g_lin_task.frame_state = FRAME_STATE_SEND;                        
                    }
                }
                
                break;
            /* 接收数据阶段，直到接收完成才处理 */
            case FRAME_STATE_RECV_DATA:
                if (UARTxOp.RxLen >= g_lin_task.lin_frame.rx_len)
                {
                    /* 最后一个字节，第一时间开接收 */
                    #if (AUTOBAUD == 1)
                    /* 关闭同步段中断 */
                    LIN_UART->ISR = (UART_ISR_SER_Msk | UART_ISR_LINSYNCF_Msk);     /* 清同步段标志 */
                    LIN_UART->IER &= ~UART_IER_LINSYNCIE_Msk;  /* 关同步段中断 */
                    
                    /* 切换波特率为初始最高波特率 */
//                    LIN_UART->CSR &= ~UART_CSR_RXEN_Msk;
                    LIN_UART->BGR = BitLength - 1;
//                    LIN_UART->CSR |= UART_CSR_RXEN_Msk;
                    #else
                    LIN_UART->CSR |= UART_CSR_RXEN_Msk;
                    #endif
                    
                }
                
                /* 接收数据 */
                /* 接收中断标志可通过读取rxreg寄存器清除 */
                tmp08 = LIN_UART->RXBUF;
                
                UARTxOp.RxBuf[UARTxOp.RxLen] = tmp08;
                UARTxOp.RxLen++;
            
                if (UARTxOp.RxLen >= g_lin_task.lin_frame.rx_len + 1)
                {
                    /* 主机主动发送，解析数据 */
                    #if (CHECKSUM_TYPE == CHECKSUM_TYPE_ENHANCED)                        
                        if (calculateChecksum(UARTxOp.RxBuf, UARTxOp.RxLen - 1) == UARTxOp.RxBuf[UARTxOp.RxLen - 1])
                        {
                            
                            /* 通过校验 */
                            if (scheduleTable[i].length != (UARTxOp.RxLen - 2))
                            {
                                lin_init_idle_state();

                                return;
                            }
                            else
                            {
                                memcpy(scheduleTable[i].data, &UARTxOp.RxBuf[1], scheduleTable[i].length);
                                scheduleTable[i].FullFlg = 0x1U;
                            }
                        }
                    #else
                        if (calculateChecksum(&UARTxOp.RxBuf[1], UARTxOp.RxLen - 2) == UARTxOp.RxBuf[UARTxOp.RxLen - 1])
                        {
                            if (scheduleTable[i].length != (UARTxOp.RxLen - 2))
                            {
                                lin_init_idle_state();

                                return;
                            }
                            else
                            {
                                memcpy(scheduleTable[i].data, &UARTxOp.RxBuf[1], scheduleTable[i].length);
                                scheduleTable[i].FullFlg = 0x1U;
                            }
                        }
                    #endif
                        lin_init_idle_state();
                }
                break;
            case FRAME_STATE_SEND:
                /* 接收数据 */
                /* 接收中断标志可通过读取rxreg寄存器清除 */
                tmp08 = LIN_UART->RXBUF;

                if (UARTxOp.TxOpc < UARTxOp.TxLen)
                {
                    if (tmp08 != UARTxOp.TxBuf[UARTxOp.TxOpc - 1])
                    {   
                        /* bit错误 */
                        #if (AUTOBAUD == 1)
                        /* 关闭同步段中断 */
                        LIN_UART->ISR = (UART_ISR_SER_Msk | UART_ISR_LINSYNCF_Msk);     /* 清同步段标志 */
                        LIN_UART->IER &= ~UART_IER_LINSYNCIE_Msk;  /* 关同步段中断 */
                        
                        /* 切换波特率为初始最高波特率 */
//                        LIN_UART->CSR &= ~UART_CSR_RXEN_Msk;
                        LIN_UART->BGR = BitLength - 1;
//                        LIN_UART->CSR |= UART_CSR_RXEN_Msk;
                        #else
                        LIN_UART->CSR |= UART_CSR_RXEN_Msk;
                        #endif
                                      
                        
                        lin_init_idle_state();
                    }
                    else
                    {
                        LIN_UART->TXBUF = UARTxOp.TxBuf[UARTxOp.TxOpc];  /* 发送一个数据 */
                        UARTxOp.TxOpc++;
                    }
                }
                else
                {
                    /* 发送完毕，快速切换至接收 */
                    #if (AUTOBAUD == 1)
                    /* 关闭同步段中断 */
                    LIN_UART->ISR = (UART_ISR_SER_Msk | UART_ISR_LINSYNCF_Msk);     /* 清同步段标志 */
                    LIN_UART->IER &= ~UART_IER_LINSYNCIE_Msk;  /* 关同步段中断 */
                    
                    /* 切换波特率为初始最高波特率 */
//                    LIN_UART->CSR &= ~UART_CSR_RXEN_Msk;
                    LIN_UART->BGR = BitLength - 1;
//                    LIN_UART->CSR |= UART_CSR_RXEN_Msk;
                    #else
                    LIN_UART->CSR |= UART_CSR_RXEN_Msk;
                    #endif
                    
                    
                    lin_init_idle_state();
                    /* 清除数据满的标志 */
                    scheduleTable[i].FullFlg = 0x0;
                }
                break;
            default:
                /* 接收数据 */
                /* 接收中断标志可通过读取rxreg寄存器清除 */
                tmp08 = LIN_UART->RXBUF;
            
                break;
        }
    }    
    
    /* 清除可能的错误中断标志 */
    if (LIN_UART->ISR & UART_ISR_PERR_Msk)
    {
        LIN_UART->ISR = UART_ISR_PERR_Msk;
    }
    if (LIN_UART->ISR & UART_ISR_OERR_Msk)
    {
        LIN_UART->ISR = UART_ISR_OERR_Msk;
    }
}

/**
  * @brief  lin UART初始化
  * @param  void
  * @retval void
  */
void lin_uart_init(void)
{
    /* 初始化uart配置 */
    Uartx_Init(LIN_UART);

    /* 中断使能 */
    #if (AUTOBAUD == 1)
    LIN_UART->MCR |= 0x1U;   /* 配置模块为LIN模式、默认为主机模式 */
    LIN_UART->MCR |= 0x2U;   /* 从机模式 */
    
    /* UART复位后处于Sleep状态，必须等待唤醒信号唤醒才会切换状态 */
    FL_UART_ClearFlag_WakeupDetect(LIN_UART);  /* 清除唤醒信号检测标志 */
    FL_UART_EnableIT_LINWK(LIN_UART);               /* 使能唤醒中断 */
    FL_UART_SetLINWkupLength(LIN_UART, 4);              /* 5*30.5=152.5us  */
    FL_UART_EnableLINWkupDetect(LIN_UART);              /* 使能唤醒检测 */
    
    FL_UART_EnableIT_LINBreak(LIN_UART);           /* 开break中断使能，等待接收break段 */
    #else
    /* 接收错误中断使能，用于同步判断 */
    FL_UART_EnableIT_RXError(LIN_UART);
    #endif
    /* 接收中断使能 */
    FL_UART_EnableIT_RXBuffFull(LIN_UART);
    FL_UART_EnableTX(LIN_UART);
    FL_UART_EnableRX(LIN_UART);
}

/**
  * @brief  lin 任务初始化
  * @param  void
  * @retval void
  */
void lin_task_init(void)
{
    uint8_t i;
    
    /* 状态机相关初始化 */
    g_lin_task.frame_state = FRAME_STATE_IDLE;
    g_lin_task.idle_timeout_cnt = LIN_IDLE_TIMEOUT;

    /* 帧数据初始化 */
    g_lin_task.lin_frame.rx_len = 0;
    g_lin_task.lin_frame.tx_len = 0;
    
    /* 根据cmd计算pid */
    for (i = 0; i < TABLE_SIZE; i++)
    {
        scheduleTable[i].pid = calculatePID(scheduleTable[i].cmd);
    }
}

#if (AUTOBAUD == 1 )
/**
  * @brief  自适应波特率初始化
  * @param  void
  * @retval void
  */
void AutoBaud_Init(void)
{

}
#endif   /*(AUTOBAUD == 1 )*/

/**
  * @brief  LIN初始化
  * @param  void
  * @retval void
  */
void lin_init(void)
{
    lin_task_init();
    
    lin_slp_pin_init();
    
    lin_uart_init();

    #if (AUTOBAUD == 1)
    AutoBaud_Init();
    #else   
    #endif
    
    lin_timeout_init();
}

#if (SLEEP_SUPPORT == 1)

/**
  * @brief  LIN休眠前初始化
  * @param  void
  * @retval void
  */
void lin_sleep_init(void)
{
    /* 清除和关闭非唤醒源中断 */
    FL_TAU_DisableIT_Update(TAU00);
    FL_TAU_ClearFlag_Update(TAU00);
    
    NVIC_DisableIRQ(TAU0_IRQn);
    NVIC_ClearPendingIRQ(TAU0_IRQn);

    
    #if (AUTOBAUD == 1)
    FL_UART_ClearFlag_WakeupDetect(LIN_UART);    /* 清除唤醒信号检测标志 */
    FL_UART_EnableIT_LINWK(LIN_UART);                 /* 使能唤醒中断 */    
    FL_UART_SetLINWkupLength(LIN_UART, 0);  /* 减小唤醒信号检测时间，以同时检测唤醒和检测 */
    FL_UART_EnableLINWkupDetect(LIN_UART);          /* 使能唤醒检测 */    
    #else
    LIN_UART->MCR |= 0x1U;   /* 配置模块为LIN模式、默认为主机模式 */
    LIN_UART->MCR |= 0x2U;   /* 从机模式 */
    
    /* UART复位后处于Sleep状态，必须等待唤醒信号唤醒才会切换状态 */
    FL_UART_ClearFlag_WakeupDetect(LIN_UART);    /* 清除唤醒信号检测标志 */
    FL_UART_EnableIT_LINWK(LIN_UART);                 /* 使能唤醒中断 */
    FL_UART_SetLINWkupLength(LIN_UART, 4);       /* 5*30.5=152.5us */
    FL_UART_EnableLINWkupDetect(LIN_UART);          /* 使能唤醒检测 */
    #endif    

    LIN_UART->ISR = LIN_UART->ISR;
    NVIC_ClearPendingIRQ(LIN_UART_IRQType);    
}

/**
  * @brief  LIN唤醒后初始化
  * @param  void
  * @retval void
  */
void lin_awake_init(void)
{
    g_lin_task.lin_frame.tx_len = 0;
    g_lin_task.lin_frame.rx_len = 0;
    g_lin_task.frame_state = FRAME_STATE_IDLE;
    g_lin_task.idle_timeout_cnt = LIN_IDLE_TIMEOUT;
    
    UARTxOp.RxLen = 0;
    UARTxOp.TxLen = 0;    
    
    #if (AUTOBAUD == 1)
    FL_UART_SetLINWkupLength(LIN_UART, 4);       /* 5*30.5=152.5us */
    #else
    /* 固定波特率唤醒，默认为接收到间隔段，超时改小便于容错 */
    g_lin_task.frame_state = FRAME_STATE_RECV_SYN;  
    /* 收到间隔段之后置起超时，等待在超时内收到同步字节0x55 */
    g_lin_task.frame_timeout_cnt = lin_calc_max_res_timeout_cnt(LIN_BIT_TIMER_LOAD, 1);
    #endif
    
    /* 使能非唤醒源中断 */
    FL_TAU_ClearFlag_Update(TAU00);      
    FL_TAU_EnableIT_Update(TAU00);  
    NVIC_EnableIRQ(TAU0_IRQn);
}

/**
  * @brief  LIN sleep判断
  * @param  void
  * @retval 1: Sleep
  *         0: Wake
  */
uint8_t lin_is_sleep(void)
{
    if (g_lin_task.frame_state == FRAME_STATE_SLEEP)
    {
        return 1;
    }
    else
    {
        return 0;
    }
}
#endif /* End #if (SLEEP_SUPPORT == 1) */

int32_t LIN_TxMsgFill(uint8_t *data, uint32_t lenth)
{
    uint32_t u32Delay = 0xFFF;
    
    /* 查询前帧是否发送完毕*/
    do
    {
        u32Delay--;
    }while((0x1 == scheduleTable[1].FullFlg) && (0 != u32Delay));
    
    /* 不足填充0xFF */
    memset(scheduleTable[1].data, 0xFF, scheduleTable[1].length);
    if (lenth > scheduleTable[1].length)
    {
        lenth = scheduleTable[1].length;
    }
    memcpy(scheduleTable[1].data, data, lenth);
    /* 置有数据标志 */
    scheduleTable[1].FullFlg = 0x1U;
    return 0;
}


