 /**    
  *******************************************************************************************************   
  * @attention    
  * Copyright (c) 2023, SHANGHAI FUDAN MICROELECTRONICS GROUP CO., LTD.(FUDAN MICROELECTRONICS./ FUDAN MICRO.)    
  * All rights reserved.    
  *    
  * Processor:                   FM33LF0xxA    
  * http:                        http://www.fmdevelopers.com.cn/    
  *    
  * Redistribution and use in source and binary forms, with or without    
  * modification, are permitted provided that the following conditions are met    
  *    
  * 1. Redistributions of source code must retain the above copyright notice,    
  *    this list of conditions and the following disclaimer.    
  *    
  * 2. Redistributions in binary form must reproduce the above copyright notice,    
  *    this list of conditions and the following disclaimer in the documentation    
  *    and/or other materials provided with the distribution.    
  *    
  * 3. Neither the name of the copyright holder nor the names of its contributors    
  *    may be used to endorse or promote products derived from this software    
  *    without specific prior written permission.    
  *    
  * 4. To provide the most up-to-date information, the revision of our documents     
  *    on the World Wide Web will be the most Current. Your printed copy may be      
  *    an earlier revision. To verify you have the latest information avaliable,    
  *    refer to: http://www.fmdevelopers.com.cn/.    
  *    
  * THIS SOFTWARE IS PROVIDED BY FUDAN MICRO "AS IS" AND ANY EXPRESSED:     
  * ORIMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES     
  * OF MERCHANTABILITY NON-INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE    
  * ARE DISCLAIMED.IN NO EVENT SHALL FUDAN MICRO OR ITS CONTRIBUTORS BE LIABLE     
  * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL     
  * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS     
  * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER    
  * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,     
  * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISINGIN ANY WAY OUT OF THE     
  * USE OF THIS SOFTWARE, EVEN IF ADVISED OFTHE POSSIBILITY OF SUCH DAMAGE.  
  *
  ******************************************************************************************************* 
  */     

#include "main.h"
#include "rmu.h"
#include "iwdt.h"
#include "svd.h"
#include "led.h"
#include "fm33lf0xxa_fl.h"
#include "pll.h"
#include "lptim16.h"
#include "uart.h"
#include "lin.h" 
#include "can.h"
#include "bootloader.h"
/**
  * @brief  HardFault 中断服务函数 请保留 
  * @param  None
  * @retval None
  */
void HardFault_Handler(void)
{   /* 软件复位MCU，使用内联函数 */
    RMU_Soft_SystemReset();
}

/**
  * @brief  掉电监控，用于执行掉电事件
  * @param  None
  * @retval None
  */
static void PowerDownMonitoring(void)
{
    /* 确认SVD监测结果是否低于阈值 */
    if (true == SVD_Result_Confirmed(SVD_BELOW_THRESHOLD, 1U))
    {   /* 防抖处理 */
        if (true == SVD_Result_Confirmed(SVD_BELOW_THRESHOLD, 2000U/*us*/))
        {
            /* 电压下降到报警阈值，处理掉电事件 */
            /* 用户程序1 */
            /* ...... */
            /* 注意：用户需确保在BOR复位之前执行完毕 */
            
            /* 确认SVD监测结果是否高于阈值，如否则持续等待 */
            while(false == SVD_Result_Confirmed(SVD_HIGHER_THRESHOLD, 20U/*us*/));
            /* 由于电源有下降到报警阈值，软件复位后重新初始化 */
            RMU_Soft_SystemReset();
        }
    }
}

static int32_t(*const TxMsgFill[BOOT_COMM_CHL])(uint8_t *, uint32_t) = 
{
    LIN_TxMsgFill, /* BOOT向LIN填充发送数据，LIN定义为通道0 */
    CAN_TxMsgFill  /* BOOT向CAN填充发送数据，CAN定义为通道1 */
};


int main(void)
{
    /* 使能IWDT */
    (void)IWDT_Init(FL_IWDT_PERIOD_4000MS);
    
    /* 延时函数初始化 */
    FL_Init();
    
    /* 使能SVD,阈值4.16V(falling)~4.284V(rising) */
    (void)SVD_Init(SVD_MONTIOR_VDD,FL_SVD_WARNING_THRESHOLD_GROUP11,FL_SVD_REFERENCE_1P0V);
    
    /* 确认SVD监测结果是否高于阈值，如否则持续等待 */
    while(false == SVD_Result_Confirmed(SVD_HIGHER_THRESHOLD, 2000U/*us*/));
    
    /* 使能BOR为2.0V */
    RMU_BOR_Init(FL_RMU_BOR_THRESHOLD_2P00V);

    /* 检查APP执行标志是否置起，如置起则直接跳转到APP */
    BOOT_Object.AppExeFlagCheck();
    
    /* 无APP，执行BOOT参数配置 */
    BOOT_Object.Configuration();
    
    /* 先将外部XTHF分频到1M，再利用PLL锁相环倍频到72M */
    (void)SelXTHFToPLL(&CMU_XTHF_Init_Data, &CMU_PLL_Init_Data);
    
    /* LED 初始化 */
    LED1_INIT();
    
    /* LPTIM16初始化 */
    LPTIM16_Init();
    
    /* Initialize all configured peripherals */
    /* SHOULD BE KEPT!!! */
    MF_Config_Init();  /* 当使用到SVD作为唤醒源时，PMU_CR只允许配置Normal Sleep模式 */
    
    /* LIN初始化 */
    lin_init();
    
    /* CAN初始化 */
    (void)Can_Controller_Init();
    
    LED1_ON();   /* 调试信息灯 */
    FL_DelayMs(50);
    LED1_OFF();  
    
    while(1)
    {
        static uint32_t Time_10ms_Cnt = 0;
        
        /* 清狗 */
        IWDT_Clr();
        
        /* 电源掉电监测处理 */
        PowerDownMonitoring();
        
        /* LIN接收帧处理 */
        if (0x1 == scheduleTable[0].FullFlg)
        {
            uint32_t primask;
            /* 中断全局变量，竞态保护 */
            primask = __get_PRIMASK();
            __disable_irq();
            scheduleTable[0].FullFlg = 0;
            __set_PRIMASK(primask);
            /* 填充到BOOT Buffer */
            BOOT_Object.RxMsgFill(scheduleTable[0].data, scheduleTable[0].length, 0/*LIN定义为通道0*/);
        }
        
        /* CAN接收帧处理 */
        if (0x1 == RxMessage.FullFlg)
        {         
            uint32_t primask;
            /* 中断全局变量，竞态保护 */
            primask = __get_PRIMASK();
            __disable_irq();
            RxMessage.FullFlg = 0;
            __set_PRIMASK(primask);
            /* 填充到BOOT Buffer */
            BOOT_Object.RxMsgFill(RxMessage.Data, RxMessage.DLC, 1/*CAN定义为通道1*/);
        }
        
        /* 功能执行 */
        BOOT_Object.Process(TxMsgFill);
        
        if (1 == Period10msFlag)
        {
            Period10msFlag = 0;
            /* 判断是否发生XTHF停振或PLL失锁事件，如有及时切回主系统时钟 */
            SwtichToMainSystemClock(&CMU_RCHF_Init_Data, &CMU_XTHF_Init_Data, &CMU_PLL_Init_Data);
            
            /* 运行指示灯 */
            if (++Time_10ms_Cnt > 50)
            {
                LED1_TOG();
                Time_10ms_Cnt = 0;
            }
        }
    }
}


